@@ -32,6 +32,10 @@
 #' @param x_arg Argument name for `x`. These are used in error
 #'   messages to inform the user about which argument has an
 #'   incompatible size.
+#'
+#' @section Dependencies:
+#' - [vec_slice()]
+#'
 #' @export
 #' @examples
 #' # Inputs with 1 observation are recycled
@@ -45,7 +49,7 @@
 #' vec_recycle_common(data.frame(x = 1), 1:5)
 #' vec_recycle_common(array(1:2, c(1, 2)), 1:5)
 #' vec_recycle_common(array(1:3, c(1, 3, 1)), 1:5)
-vec_recycle <- function(x, size, ..., x_arg = "x") {
+vec_recycle <- function(x, size, ..., x_arg = "") {
   if (!missing(...)) {
     ellipsis::check_dots_empty()
   }
