@@ -1,3 +1,7 @@
+#ifndef VCTRS_H
+#define VCTRS_H
+
+
 #define R_NO_REMAP
 #include <R.h>
 #include <Rinternals.h>
@@ -6,11 +10,28 @@
 #include <stdbool.h>
 #include <stdint.h>
 
-typedef R_xlen_t r_ssize_t;
+
+extern bool vctrs_debug_verbose;
 
 #define VCTRS_ASSERT(condition) ((void)sizeof(char[1 - 2*!(condition)]))
 
 
+typedef R_xlen_t r_ssize;
+#define R_SSIZE_MAX R_XLEN_T_MAX
+
+#define r_length Rf_xlength
+#define r_new_vector Rf_allocVector
+
+// An ERR indicates either a C NULL in case of no error, or a
+// condition object otherwise
+#define ERR SEXP
+
+// Utilities for deciding whether or not to re-encode as UTF-8
+#define CHAR_IS_UTF8(x)  (LEVELS(x) & 8)
+#define CHAR_IS_ASCII(x) (LEVELS(x) & 64)
+#define CHAR_NEEDS_REENCODE(x) (!(CHAR_IS_ASCII(x) || (x) == NA_STRING || CHAR_IS_UTF8(x)))
+#define CHAR_REENCODE(x) Rf_mkCharCE(Rf_translateCharUTF8(x), CE_UTF8)
+
 // Vector types -------------------------------------------------
 
 enum vctrs_type {
@@ -86,6 +107,7 @@ struct vctrs_proxy_info vec_proxy_info(SEXP x);
 enum vctrs_type vec_typeof(SEXP x);
 enum vctrs_type vec_proxy_typeof(SEXP x);
 const char* vec_type_as_str(enum vctrs_type type);
+bool vec_is_list(SEXP x);
 bool vec_is_vector(SEXP x);
 bool vec_is_partial(SEXP x);
 
@@ -325,6 +347,8 @@ extern SEXP vctrs_shared_true;
 extern SEXP vctrs_shared_false;
 
 extern Rcomplex vctrs_shared_na_cpl;
+extern SEXP vctrs_shared_na_lgl;
+extern SEXP vctrs_shared_na_list;
 
 SEXP vec_unspecified(R_len_t n);
 bool vec_is_unspecified(SEXP x);
@@ -334,121 +358,97 @@ bool vec_is_unspecified(SEXP x);
 
 #include "arg.h"
 #include "names.h"
+#include "owned.h"
 
 enum vctrs_proxy_kind {
-  vctrs_proxy_default,
-  vctrs_proxy_equal,
-  vctrs_proxy_compare
+  VCTRS_PROXY_KIND_default,
+  VCTRS_PROXY_KIND_equal,
+  VCTRS_PROXY_KIND_compare,
+  VCTRS_PROXY_KIND_order,
+  VCTRS_PROXY_KIND_complete
 };
 
 SEXP vec_proxy(SEXP x);
 SEXP vec_proxy_equal(SEXP x);
-SEXP vec_proxy_recursive(SEXP x, enum vctrs_proxy_kind kind);
-SEXP vec_restore(SEXP x, SEXP to, SEXP i);
+SEXP vec_proxy_compare(SEXP x);
+SEXP vec_proxy_order(SEXP x);
+SEXP vec_proxy_complete(SEXP x);
+SEXP vec_restore(SEXP x, SEXP to, SEXP n, const enum vctrs_owned owned);
+SEXP vec_restore_default(SEXP x, SEXP to, const enum vctrs_owned owned);
 R_len_t vec_size(SEXP x);
 R_len_t vec_size_common(SEXP xs, R_len_t absent);
-SEXP vec_dim(SEXP x);
-R_len_t vec_dim_n(SEXP x);
-SEXP vec_bare_dim(SEXP x);
-R_len_t vec_bare_dim_n(SEXP x);
-SEXP vec_cast(SEXP x, SEXP to, struct vctrs_arg* x_arg, struct vctrs_arg* to_arg);
 SEXP vec_cast_common(SEXP xs, SEXP to);
-SEXP vec_coercible_cast(SEXP x, SEXP to, struct vctrs_arg* x_arg, struct vctrs_arg* to_arg);
-SEXP vec_slice(SEXP x, SEXP index);
+SEXP vec_slice(SEXP x, SEXP subscript);
+SEXP vec_slice_impl(SEXP x, SEXP subscript);
 SEXP vec_chop(SEXP x, SEXP indices);
 SEXP vec_slice_shaped(enum vctrs_type type, SEXP x, SEXP index);
-SEXP vec_assign(SEXP x, SEXP index, SEXP value);
+SEXP vec_proxy_assign(SEXP proxy, SEXP index, SEXP value);
 bool vec_requires_fallback(SEXP x, struct vctrs_proxy_info info);
 SEXP vec_init(SEXP x, R_len_t n);
-SEXP vec_type(SEXP x);
+SEXP vec_ptype(SEXP x, struct vctrs_arg* x_arg);
 SEXP vec_ptype_finalise(SEXP x);
 bool vec_is_unspecified(SEXP x);
 SEXP vec_recycle(SEXP x, R_len_t size, struct vctrs_arg* x_arg);
+SEXP vec_recycle_fallback(SEXP x, R_len_t size, struct vctrs_arg* x_arg);
 SEXP vec_recycle_common(SEXP xs, R_len_t size);
 SEXP vec_names(SEXP x);
+SEXP vec_proxy_names(SEXP x);
 SEXP vec_group_loc(SEXP x);
-SEXP vec_match(SEXP needles, SEXP haystack);
+SEXP vec_identify_runs(SEXP x);
+SEXP vec_match_params(SEXP needles, SEXP haystack, bool na_equal,
+                      struct vctrs_arg* needles_arg, struct vctrs_arg* haystack_arg);
+
+#include "cast.h"
+static inline SEXP vec_cast(SEXP x, SEXP to, struct vctrs_arg* x_arg, struct vctrs_arg* to_arg) {
+  struct cast_opts opts = {
+    .x = x,
+    .to = to,
+    .x_arg = x_arg,
+    .to_arg = to_arg
+  };
+  return vec_cast_opts(&opts);
+}
+
+static inline SEXP vec_match(SEXP needles, SEXP haystack) {
+  return vec_match_params(needles, haystack, true, NULL, NULL);
+}
+
 
 SEXP vec_c(SEXP xs,
            SEXP ptype,
            SEXP name_spec,
            const struct name_repair_opts* name_repair);
 
-SEXP vec_type2(SEXP x,
-               SEXP y,
-               struct vctrs_arg* x_arg,
-               struct vctrs_arg* y_arg,
-               int* left);
-
-SEXP vec_ptype2_dispatch(SEXP x, SEXP y,
-                         enum vctrs_type x_type,
-                         enum vctrs_type y_type,
-                         struct vctrs_arg* x_arg,
-                         struct vctrs_arg* y_arg,
-                         int* left);
-
-SEXP vec_ptype2_dispatch_s3(SEXP x,
-                            SEXP y,
-                            struct vctrs_arg* x_arg,
-                            struct vctrs_arg* y_arg);
-
-SEXP vec_cast_dispatch(SEXP x,
-                       SEXP to,
-                       enum vctrs_type x_type,
-                       enum vctrs_type to_type,
-                       bool* lossy,
-                       struct vctrs_arg* x_arg,
-                       struct vctrs_arg* to_arg);
-
-SEXP df_ptype2(SEXP x, SEXP y, struct vctrs_arg* x_arg, struct vctrs_arg* y_arg);
-SEXP df_as_dataframe(SEXP x, SEXP to, struct vctrs_arg* x_arg, struct vctrs_arg* to_arg);
-
 bool is_data_frame(SEXP x);
-bool is_record(SEXP x);
 
 R_len_t df_size(SEXP x);
 R_len_t df_rownames_size(SEXP x);
 R_len_t df_raw_size(SEXP x);
 R_len_t df_raw_size_from_list(SEXP x);
-SEXP vec_bare_df_restore(SEXP x, SEXP to, SEXP n);
-SEXP vec_df_restore(SEXP x, SEXP to, SEXP n);
-
-SEXP chr_assign(SEXP out, SEXP index, SEXP value, bool clone);
-SEXP list_assign(SEXP out, SEXP index, SEXP value, bool clone);
-SEXP df_assign(SEXP out, SEXP index, SEXP value, bool clone);
+SEXP vec_bare_df_restore(SEXP x, SEXP to, SEXP n, const enum vctrs_owned owned);
+SEXP vec_df_restore(SEXP x, SEXP to, SEXP n, const enum vctrs_owned owned);
 
 // equal_object() never propagates missingness, so
 // it can return a `bool`
 bool equal_object(SEXP x, SEXP y);
+bool equal_object_normalized(SEXP x, SEXP y);
 bool equal_names(SEXP x, SEXP y);
 
-/**
- * These functions are meant to be used in loops so it is the callers
- * responsibility to ensure that:
- *
- * - `x` and `y` have identical SEXTYPEs
- * - `i` is a valid index into `x`, and `j` is a valid index into `y`.
- *
- * The behaviour is undefined if these conditions are not true.
- */
-int equal_scalar(SEXP x, R_len_t i, SEXP y, R_len_t j, bool na_equal);
 int compare_scalar(SEXP x, R_len_t i, SEXP y, R_len_t j, bool na_equal);
 
 uint32_t hash_object(SEXP x);
-void hash_fill(uint32_t* p, R_len_t n, SEXP x);
+void hash_fill(uint32_t* p, R_len_t n, SEXP x, bool na_equal);
 
 SEXP vec_unique(SEXP x);
 bool duplicated_any(SEXP names);
 
-// Rowwise operations -------------------------------------------
+// Data frame column iteration ----------------------------------
 
 // Used in functions that treat data frames as vectors of rows, but
-// iterate over them column wise. Examples are `vec_equal()` and
+// iterate over columns. Examples are `vec_equal()` and
 // `vec_compare()`.
 
 /**
- * @member out A vector of size `n_row` containing the output of the
- *   row wise data frame operation.
  * @member row_known A boolean array of size `n_row`. Allocated on the R heap.
  *   Initially, all values are initialized to `false`. As we iterate along the
  *   columns, we flip the corresponding row's `row_known` value to `true` if we
@@ -460,20 +460,58 @@ bool duplicated_any(SEXP names);
  * @member remaining The number of `row_known` values that are still `false`.
  *   If this hits `0` before we traverse the entire data frame, we can exit
  *   immediately because all `out` values are already known.
+ * @member size The number of rows in the data frame.
  */
-struct vctrs_df_rowwise_info {
-  SEXP out;
+struct df_short_circuit_info {
   SEXP row_known;
   bool* p_row_known;
+  PROTECT_INDEX row_known_pi;
   R_len_t remaining;
+  R_len_t size;
 };
 
-#define PROTECT_DF_ROWWISE_INFO(info, n) do {  \
-  PROTECT((info)->out);                        \
-  PROTECT((info)->row_known);                  \
-  *n += 2;                                     \
+#define PROTECT_DF_SHORT_CIRCUIT_INFO(p_info, p_n) do {             \
+  PROTECT_WITH_INDEX((p_info)->row_known, &(p_info)->row_known_pi); \
+  *(p_n) += 1;                                                      \
 } while (0)
 
+static inline struct df_short_circuit_info new_df_short_circuit_info(R_len_t size, bool lazy) {
+  SEXP row_known;
+  bool* p_row_known;
+
+  if (lazy) {
+    row_known = PROTECT(R_NilValue);
+    p_row_known = NULL;
+  } else {
+    row_known = PROTECT(Rf_allocVector(RAWSXP, size * sizeof(bool)));
+    p_row_known = (bool*) RAW(row_known);
+
+    // To begin with, no rows have a known comparison value
+    memset(p_row_known, false, size * sizeof(bool));
+  }
+
+  struct df_short_circuit_info info = {
+    .row_known = row_known,
+    .p_row_known = p_row_known,
+    .remaining = size,
+    .size = size
+  };
+
+  UNPROTECT(1);
+  return info;
+}
+
+static inline void init_lazy_df_short_circuit_info(struct df_short_circuit_info* p_info) {
+  if (p_info->row_known != R_NilValue) {
+    return;
+  }
+
+  p_info->row_known = Rf_allocVector(RAWSXP, p_info->size * sizeof(bool));
+  REPROTECT(p_info->row_known, p_info->row_known_pi);
+
+  p_info->p_row_known = (bool*) RAW(p_info->row_known);
+}
+
 // Missing values -----------------------------------------------
 
 // Annex F of C99 specifies that `double` should conform to the IEEE 754
@@ -512,32 +550,33 @@ enum vctrs_dbl_class dbl_classify(double x);
 
 // Factor methods -----------------------------------------------
 
-SEXP fct_ptype2(SEXP x, SEXP y, struct vctrs_arg* x_arg, struct vctrs_arg* y_arg);
-SEXP ord_ptype2(SEXP x, SEXP y, struct vctrs_arg* x_arg, struct vctrs_arg* y_arg);
+SEXP chr_as_factor(SEXP x, SEXP to, bool* lossy, struct vctrs_arg* to_arg);
+SEXP chr_as_ordered(SEXP x, SEXP to, bool* lossy, struct vctrs_arg* to_arg);
 
 SEXP fct_as_character(SEXP x, struct vctrs_arg* x_arg);
-SEXP ord_as_character(SEXP x, struct vctrs_arg* x_arg);
-
-SEXP chr_as_factor(SEXP x, SEXP to, bool* lossy, struct vctrs_arg* to_arg);
 SEXP fct_as_factor(SEXP x, SEXP to, bool* lossy, struct vctrs_arg* x_arg, struct vctrs_arg* to_arg);
 
-SEXP chr_as_ordered(SEXP x, SEXP to, bool* lossy, struct vctrs_arg* to_arg);
-SEXP ord_as_ordered(SEXP x, SEXP to, bool* lossy, struct vctrs_arg* x_arg, struct vctrs_arg* to_arg);
+SEXP ord_as_character(SEXP x, struct vctrs_arg* x_arg);
 
 // Datetime methods ---------------------------------------------
 
+SEXP date_as_date(SEXP x);
+SEXP date_as_posixct(SEXP x, SEXP to);
+SEXP date_as_posixlt(SEXP x, SEXP to);
+SEXP posixct_as_date(SEXP x, bool* lossy);
+SEXP posixlt_as_date(SEXP x, bool* lossy);
+SEXP posixct_as_posixct(SEXP x, SEXP to);
+SEXP posixlt_as_posixct(SEXP x, SEXP to);
+SEXP posixct_as_posixlt(SEXP x, SEXP to);
+SEXP posixlt_as_posixlt(SEXP x, SEXP to);
+
+SEXP vec_date_restore(SEXP x, SEXP to, const enum vctrs_owned owned);
+SEXP vec_posixct_restore(SEXP x, SEXP to, const enum vctrs_owned owned);
+SEXP vec_posixlt_restore(SEXP x, SEXP to, const enum vctrs_owned owned);
+
 SEXP date_datetime_ptype2(SEXP x, SEXP y);
 SEXP datetime_datetime_ptype2(SEXP x, SEXP y);
 
-// Tibble methods ----------------------------------------------
-
-SEXP tibble_ptype2(SEXP x, SEXP y, struct vctrs_arg* x_arg, struct vctrs_arg* y_arg);
-
-// Character translation ----------------------------------------
-
-SEXP obj_maybe_translate_encoding(SEXP x, R_len_t size);
-SEXP obj_maybe_translate_encoding2(SEXP x, R_len_t x_size, SEXP y, R_len_t y_size);
-
 // Growable vector ----------------------------------------------
 
 struct growable {
@@ -572,25 +611,28 @@ static inline void growable_push_int(struct growable* g, int i) {
 
 #define UNPROTECT_GROWABLE(g) do { UNPROTECT(1);} while(0)
 
-
-// Shape --------------------------------------------------------
-
-bool has_dim(SEXP x);
-
-
 // Conditions ---------------------------------------------------
 
-void vctrs_stop_unsupported_type(enum vctrs_type, const char* fn) __attribute__((noreturn));
 void stop_scalar_type(SEXP x, struct vctrs_arg* arg) __attribute__((noreturn));
 void vec_assert(SEXP x, struct vctrs_arg* arg);
+__attribute__((noreturn))
 void stop_incompatible_size(SEXP x, SEXP y,
                             R_len_t x_size, R_len_t y_size,
                             struct vctrs_arg* x_arg,
-                            struct vctrs_arg* y_arg)
-  __attribute__((noreturn));
+                            struct vctrs_arg* y_arg);
+__attribute__((noreturn))
+void stop_incompatible_type(SEXP x,
+                            SEXP y,
+                            struct vctrs_arg* x_arg,
+                            struct vctrs_arg* y_arg,
+                            bool cast);
+__attribute__((noreturn))
 void stop_recycle_incompatible_size(R_len_t x_size, R_len_t size,
-                                    struct vctrs_arg* x_arg)
-  __attribute__((noreturn));
+                                    struct vctrs_arg* x_arg);
+__attribute__((noreturn))
+void stop_incompatible_shape(SEXP x, SEXP y,
+                             R_len_t x_size, R_len_t y_size, int axis,
+                             struct vctrs_arg* p_x_arg, struct vctrs_arg* p_y_arg);
 void stop_corrupt_factor_levels(SEXP x, struct vctrs_arg* arg) __attribute__((noreturn));
 void stop_corrupt_ordered_levels(SEXP x, struct vctrs_arg* arg) __attribute__((noreturn));
 
@@ -603,4 +645,10 @@ void stop_corrupt_ordered_levels(SEXP x, struct vctrs_arg* arg) __attribute__((n
 # define COMPLEX_RO(x) ((const Rcomplex*) COMPLEX(x))
 # define STRING_PTR_RO(x) ((const SEXP*) STRING_PTR(x))
 # define RAW_RO(x) ((const Rbyte*) RAW(x))
+# define DATAPTR_RO(x) ((const void*) STRING_PTR(x))
+#endif
+
+#define VECTOR_PTR_RO(x) ((const SEXP*) DATAPTR_RO(x))
+
+
 #endif
