@@ -1,12 +1,15 @@
-context("test-names")
 
 # vec_names() ---------------------------------------------------------
 
 test_that("vec_names() retrieves names", {
   expect_null(vec_names(letters))
   expect_identical(vec_names(set_names(letters)), letters)
-  expect_null(vec_names(mtcars))
+
+  expect_identical(vec_names(mtcars), row.names(mtcars))
+  expect_null(vec_names(unrownames(mtcars)))
+
   expect_identical(vec_names(Titanic), dimnames(Titanic)[[1]])
+
   x <- matrix(1L, dimnames = list("row", "col"))
   expect_identical(vec_names(x), dimnames(x)[[1]])
 })
@@ -18,23 +21,6 @@ test_that("vec_names() dispatches", {
   expect_identical(vec_names(foobar()), "dispatched!")
 })
 
-test_that("vec_names<- sets names", {
-  x <- letters
-  vec_names(x) <- letters
-  expect_identical(vec_names(x), letters)
-  vec_names(x) <- NULL
-  expect_null(vec_names(x))
-
-  y <- iris
-  vec_names(y) <- as.character(-seq_len(vec_size(y)))
-  expect_identical(row.names(y), row.names(iris))
-  expect_null(vec_names(y))
-
-  z <- ones(3, 2, 1)
-  vec_names(z) <- as.character(1:3)
-  expect_identical(vec_names(z), as.character(1:3))
-})
-
 
 # vec_names2() -------------------------------------------------------------
 
@@ -45,8 +31,13 @@ test_that("vec_names2() repairs names", {
 })
 
 test_that("vec_names2() treats data frames and arrays as vectors", {
-  expect_identical(vec_names2(mtcars), rep_len("", nrow(mtcars)))
+  expect_identical(vec_names2(mtcars), row.names(mtcars))
   expect_identical(vec_names2(as.matrix(mtcars)), row.names(mtcars))
+
+  df <- unrownames(mtcars)
+  exp <- rep_len("", nrow(mtcars))
+  expect_identical(vec_names2(df), exp)
+  expect_identical(vec_names2(as.matrix(df)), exp)
 })
 
 test_that("vec_names2() accepts and checks repair function", {
@@ -66,6 +57,11 @@ test_that("vec_as_names() requires character vector", {
   expect_error(vec_as_names(NULL), "`names` must be a character vector")
 })
 
+test_that("vec_as_names() validates `repair`", {
+  expect_error(vec_as_names("x", repair = "foo"), "can't be \"foo\"")
+  expect_error(vec_as_names(1, repair = 1), "string or a function")
+})
+
 test_that("vec_as_names() repairs names", {
   expect_identical(vec_as_names(chr(NA, NA)), c("", ""))
   expect_identical(vec_as_names(chr(NA, NA), repair = "unique"), c("...1", "...2"))
@@ -113,6 +109,19 @@ test_that("vec_as_names() repairs names before invoking repair function", {
   expect_identical(vec_as_names(chr(NA, NA), repair = identity), c("", ""))
 })
 
+test_that("vec_as_names() is noisy by default", {
+  verify_output(test_path("output", "test-vec-as-names.txt"), {
+    # Noisy name repair
+    vec_as_names(c("x", "x"), repair = "unique")
+
+    # Quiet name repair
+    vec_as_names(c("x", "x"), repair = "unique", quiet = TRUE)
+
+    # Hint at repair argument, if known
+    vec_as_names(c("x", "x"), repair = "check_unique", repair_arg = "repair")
+  })
+})
+
 test_that("validate_minimal_names() checks names", {
   expect_error(validate_minimal_names(1), "must return a character vector")
   expect_error(validate_minimal_names(NULL), "can't return `NULL`")
@@ -138,6 +147,16 @@ test_that("vec_as_names_validate() validates repair arguments", {
   )
 })
 
+test_that("vec_as_names() is quiet when function is supplied (#1018)", {
+  expect_silent(
+    vctrs::vec_as_names(
+      c("a", "b"),
+      repair = function(x) paste0(x, "a"),
+      quiet = FALSE
+    )
+  )
+})
+
 
 # vec_repair_names() -------------------------------------------------------
 
@@ -176,14 +195,32 @@ test_that("vec_set_names() sets matrix/array names", {
   expect_equal(vec_set_names(y, names), exp)
 })
 
-test_that("vec_set_names() does not set row names on data frames", {
-  x <- data.frame(a = 1, b = 2)
-  expect_equal(vec_set_names(x, "r1"), x)
+test_that("vec_set_names() doesn't alter names", {
+  x <- matrix(1, dimnames = list(rows = "a", cols = "x"))
+  vec_set_names(x, "y")
+  expect_equal(vec_names2(x), "a")
+  expect_equal(colnames(x), "x")
+  vec_set_names(x, NULL)
+  expect_equal(vec_names2(x), "a")
+  expect_equal(colnames(x), "x")
+
+  y <- array(1:4, dim = c(1, 2, 2), dimnames = list(rows = "a", one = 1:2, two = 1:2))
+  vec_set_names(y, "y")
+  expect_equal(vec_names2(y), "a")
+  vec_set_names(y, NULL)
+  expect_equal(vec_names2(y), "a")
 })
 
-test_that("vec_set_names() leaves existing data frame row names intact", {
-  x <- data.frame(a = 1, b = 2, row.names = "original")
-  expect_equal(rownames(vec_set_names(x, "new")), "original")
+test_that("vec_set_names() sets row names on data frames", {
+  expect_identical(
+    vec_set_names(data_frame(x = 1), "foo"),
+    new_data_frame(list(x = 1), row.names = "foo")
+  )
+
+  expect_identical(
+    vec_set_names(data_frame(x = 1:2), c("foo", "foo")),
+    new_data_frame(list(x = 1:2), row.names = c("foo...1", "foo...2"))
+  )
 })
 
 test_that("vec_set_names() correctly sets names on POSIXlt objects", {
@@ -236,6 +273,12 @@ test_that("vec_set_names() errors with bad `names`", {
   expect_error(vec_set_names(1, c("x", "y")), "The size of `names`, 2")
 })
 
+test_that("vec_names() and vec_set_names() work with 1-dimensional arrays", {
+  x <- array(1:2, dimnames = list(c("a", "b")))
+  expect_identical(vec_names(x), c("a", "b"))
+  expect_identical(vec_names(vec_set_names(x, c("A", "B"))), c("A", "B"))
+})
+
 # minimal names -------------------------------------------------------------
 
 test_that("minimal names are made from `n` when `name = NULL`", {
@@ -262,8 +305,13 @@ test_that("as_minimal_names() is idempotent", {
 })
 
 test_that("minimal_names() treats data frames and arrays as vectors", {
-  expect_identical(minimal_names(mtcars), rep_len("", nrow(mtcars)))
+  expect_identical(minimal_names(mtcars), row.names(mtcars))
   expect_identical(minimal_names(as.matrix(mtcars)), row.names(mtcars))
+
+  df <- unrownames(mtcars)
+  exp <- rep_len("", nrow(mtcars))
+  expect_identical(minimal_names(df), exp)
+  expect_identical(minimal_names(as.matrix(df)), exp)
 })
 
 test_that("as_minimal_names() copies on write", {
@@ -681,28 +729,28 @@ test_that("Name repair works with non-UTF-8 names", {
 
 test_that("names cannot be empty", {
   expect_error_cnd(
-    stop_names_cannot_be_empty(1:3),
+    stop_names_cannot_be_empty(c("", "")),
     class = c("vctrs_error_names_cannot_be_empty", "vctrs_error_names", "vctrs_error"),
-    message = "Names must not be empty.",
-    locations = 1:3
+    message = "Names can't be empty.",
+    names = c("", "")
   )
 })
 
 test_that("names cannot be dot dot", {
   expect_error_cnd(
-    stop_names_cannot_be_dot_dot(1:3),
+    stop_names_cannot_be_dot_dot(c("..1", "..2")),
     class = c("vctrs_error_names_cannot_be_dot_dot", "vctrs_error_names", "vctrs_error"),
-    message = "Names must not be of the form `...` or `..j`.",
-    locations = 1:3
+    message = "Names can't be of the form `...` or `..j`.",
+    names = c("..1", "..2")
   )
 })
 
 test_that("names must be unique", {
   expect_error_cnd(
-    stop_names_must_be_unique(1:3),
+    stop_names_must_be_unique(c("x", "y", "y", "x")),
     class = c("vctrs_error_names_must_be_unique", "vctrs_error_names", "vctrs_error"),
     message = "Names must be unique.",
-    locations = 1:3
+    names = c("x", "y", "y", "x")
   )
 })
 
@@ -730,6 +778,10 @@ test_that("NULL name specs works with scalars", {
   expect_identical(apply_name_spec(NULL, "foo", NULL, 1L), "foo")
   expect_named(vec_c(foo = 1), "foo")
 
+  expect_identical(apply_name_spec(NULL, "foo", chr(), 0L), chr())
+  expect_named(vec_c(foo = set_names(dbl())), chr())
+  expect_named(vec_c(foo = set_names(dbl()), bar = set_names(dbl())), chr())
+
   expect_error(apply_name_spec(NULL, "foo", c("a", "b")), "vector of length > 1")
   expect_error(vec_c(foo = c(a = 1, b = 2)), "vector of length > 1")
 
@@ -767,3 +819,36 @@ test_that("can pass glue string as name spec", {
 test_that("`outer` is recycled before name spec is invoked", {
   expect_identical(vec_c(outer = 1:2, .name_spec = "{outer}"), c(outer = 1L, outer = 2L))
 })
+
+test_that("apply_name_spec() recycles return value not arguments (#1099)", {
+  out <- unstructure(apply_name_spec("foo", "outer", c("a", "b", "c")))
+  expect_identical(out, c("foo", "foo", "foo"))
+
+  inner <- NULL
+  outer <- NULL
+  spec <- function(outer, inner) {
+    inner <<- inner
+    outer <<- outer
+  }
+
+  apply_name_spec(spec, "outer", c("a", "b", "c"))
+  expect_identical(inner, c("a", "b", "c"))
+  expect_identical(outer, "outer")
+})
+
+test_that("r_chr_paste_prefix() works", {
+  nms <- c("foo", "bar")
+
+  expect_equal(
+    .Call(vctrs_chr_paste_prefix, nms, "baz", "."),
+    c("baz.foo", "baz.bar")
+  )
+
+  # Greater than `VCTRS_PASTE_BUFFER_MAX_SIZE`
+  long_prefix <- strrep("a", 5000)
+
+  expect_equal(
+    .Call(vctrs_chr_paste_prefix, nms, long_prefix, "."),
+    paste0(long_prefix, ".", nms)
+  )
+})
